#include "obot.h"

static int ERROR_CODE = 1;

BOOL setting(HEXADECIMAL_FIELDS *h, char *e, int b, int f, int l, int s) {
    int size;

    strcpy(h->file_name, e);
    h->buffer_size = b;
    h->field_count = f;
    h->limit = l;
    h->separator = s;

    h->whole = file_size(h->file_name);
    if(h->whole == FILE_ERROR) {
        h->buffer = NULL;
    } else {
        size = (int)h->whole;
        if(h->buffer_size > size) {
            h->buffer_size = size;
        }

        h->buffer = malloc(h->buffer_size);
        if(h->buffer == NULL) {
            fprintf(stderr, "malloc() error: %d bytes", h->buffer_size);
            return FALSE;
        }

    }

    return TRUE;
}

int represented_in_hexadecimal(HEXADECIMAL_FIELDS *h) {
    char field[4] = {0, 0, 0, 0};
    int exit_code = 0;
    int k = 0;
    int m, n;
    unsigned char uc;
    FILE *file = fopen(h->file_name, "rb");
    if(file == NULL) {
        return ERROR_CODE;
    }

    fread(h->buffer, 1, h->buffer_size, file);
    h->line_count = 0; 
    while(k < h->buffer_size) {
        for(m = 0; m < h->field_count; m++) {
            uc = (unsigned char)h->buffer[k++];
            n = (int)uc;
            sprintf(field, "%02X%c", n, h->separator);
            fputs(field, stdout);

            if(uc == 0x0A) {
                h->line_count += 1;
                if(h->line_count >= h->limit) {
                    k = h->buffer_size;
                }

            }

            if(k >= h->buffer_size) {
                break;
            }

        }

        fputc('\n', stdout);
    }

    fclose(file);
    return exit_code;
}

long file_size(char *file_name) {
    long size = FILE_ERROR;
    FILE *p = fopen(file_name, "rb");
    if(p == NULL) {
        error_handler("fopen() error:", file_name);
        return size;
    }

    fseek(p, 0, END_OF_FILE);
    size = ftell(p);
    if( size < 0L ) {
        error_handler("ftell() error:", file_name);
    }

    fclose(p);
    return size;
}

/*
1: obot.exe       file
2: 640            buffer
3: 3              lines
4: 20h            separator
5: 20             fields
6: 2147483647     rows
*/

void acquire_parameters_from_text(const char *parameters_text, char *file_name, int *buffer_size, int *limit, int *separator, int *field_count, int *maximum_row_number) {
    const int HEXADECIMAL_NUMBER = 16;

    char *p;
    char *stop;
    char c[C_SIZE];
    FILE *setting = fopen(parameters_text, "rt");
    int k;
    int line_number = 0;

    if(setting != NULL) {
        for(;;) {
            p = fgets(c, C_SIZE, setting);
            if(p == NULL) {
                break;
            } else {
                k = strcspn(c, "\n'|\t");
                c[k] = '\0';
            }

            line_number += 1;
            switch(line_number) {
            case 1:
                strcpy(file_name, c); /* a binary or text file */
                break;
            case 2:
                *buffer_size = (int)strtol(c, &stop, 10); /* 640 bytes */
                break;
            case 3:
                *limit = (int)strtol(c, &stop, 10); /* 3 lines of the text file */
                break;
            case 4:
                *separator = (int)strtol(c, &stop, HEXADECIMAL_NUMBER); /* 20h */
                break;
            case 5:
                *field_count = (int)strtol(c, &stop, 10); /* The table has 20 fields or less. */
                break;
            case 6:
                *maximum_row_number = (int)strtol(c, &stop, 10); /* The table has 2147483647 rows or less. */
                break;
            default:
                break;
            } /* switch(line_number) */

        } /* for(;;) */

        fclose(setting);
    } /* if(setting != NULL) */

}

void error_handler(const char *p, char *q) {
    FILE *file = fopen(ERROR_TEXT, "wt");
    fprintf(file, "%s %s", p, q);;
    fclose(file);
}

void release(HEXADECIMAL_FIELDS *h) {
    free(h->buffer);
}
